==> phone-1.0.4a.tar.gz: phone-1.0.4a/communicate.c <==
#include    <netdb.h>
#include    <signal.h>
#include    <stdio.h>
#include    <stdlib.h>
#include    <time.h>
#include    <netinet/in.h>
#include    <sys/types.h>
#include    <sys/socket.h>
#include    <sys/ioctl.h>
#include    <linux/soundcard.h>
#include    <openssl/opensslconf.h>
#include    <openssl/des.h>

#include    "defs.h"
#include    "sock_rw.h"
#include    "dsp.h"
#include    "compress.h"

void        print_time(FILE*fp)
{
  char        buf[64];
  time_t      t;
  int         len;

  /*  */
  time(&t);

  /* ʸѴ */
  ctime_r(&t, buf);

  /* ʸԤʤоõ */
  len = strlen(buf);
  if('\n' == buf[len - 1])
    buf[len - 1] = '\0';

  /* 񤭽Ф */
  fprintf(fp, "%s", buf);
}

void        rcv(int sockfd, OPTION*opt)
{
  char*       buf;  /* buffer for receiving */
  char*       cry;  /* buffer for decryption */
  char*       cmp;  /* buffer for decompression */
  char*       ptr;
  FILE*       snd;
  int         count = 0;
  int         len;
  int         max_len;
  const_des_cblock key = DES_KEY;

  max_len = opt->freq * opt->channels * opt->bits / 8 * opt->interval / 1000;

  buf = malloc(HED_LEN + max(MSG_LEN, max_len) + 1);
  if(NULL == buf){
    fprintf(stderr, "malloc() failure.\n");
    exit(EXIT_FAILURE);
  }

  /* header receive header */
  *buf = 0;
  if(readn(sockfd, buf, HED_LEN) < 0)
    fprintf(stderr, "error\n");

  /* lengthФ */
  len  =  buf[1] & 0xff;
  len |= (buf[2] & 0xff) <<  8;
  len |= (buf[3] & 0xff) << 16;
  len |= (buf[4] & 0xff) << 24;

  ptr = buf + HED_LEN;

  /* Ĥ receive remaing data */
  if(readn(sockfd, ptr, len) < 0)
    fprintf(stderr, "error\n");

  /* δĶ򥻥å */
  opt->freq = atoi(ptr);
  while(' ' != *ptr++);
  opt->channels = atoi(ptr);
  while(' ' != *ptr++);
  opt->bits = atoi(ptr);
  while(' ' != *ptr++);
  opt->interval = atoi(ptr);

print_options(stderr, opt);

  max_len = opt->freq * opt->channels * opt->bits / 8 * opt->interval / 1000;
  free(buf);
  buf = malloc(HED_LEN + max(MSG_LEN, max_len) + 1);
  cry = malloc(max(MSG_LEN, max_len) + 1);
  cmp = malloc(max(MSG_LEN, max_len) + 1);
  if(NULL == buf || NULL == cry || NULL == cmp){
    fprintf(stderr, "malloc() failure.\n");
    exit(EXIT_FAILURE);
  }

  /* open & initialize sound device (write mode) */
  snd = init_dsp("/dev/dsp", "w", opt->channels, opt->freq, opt->bits);

  fflush(stdin);
  fflush(stdout);

  while(1){
    /* header receive header */
    *buf = 0;
    if(readn(sockfd, buf, HED_LEN) < 0)
      fprintf(stderr, "error\n");

    /* lengthФ */
    len  =  buf[1] & 0xff;
    len |= (buf[2] & 0xff) <<  8;
    len |= (buf[3] & 0xff) << 16;
    len |= (buf[4] & 0xff) << 24;

    ptr = buf + HED_LEN;

    /* Ĥ receive remaing data */
    if(readn(sockfd, ptr, len) < 0)
      fprintf(stderr, "error\n");

    /* Ź沽Ƥ椹 decrypt if encrypted */
    if(*buf & ENCRYPTED){
      my_decrypt(key, ptr, cry, max_len);
      ptr = cry;
    }

    /* ̤Ƥвह decompress if compressed */
    if(*buf & COMPRESSED){
      decomp(ptr, len, cmp, max_len);
      ptr = cmp;
    }

    /* packet analize packet */
    if(*buf & SOUND)
      output_buf(snd, ptr, max_len, opt->channels, opt->bits);
    else if(*buf & MESSAGE){
      fprintf(stderr,
              "\033[2K"   /* ΤԤõ delete current line */
              "\n"        /*  newline */
              "\033M"     /* 1Ծ˰ư */
              "\033[7m"); /* ȿžɽ reversed indication */

      /* ߤλɽ print current time */
      print_time(stderr);

      fprintf(stderr,
              " massage received: ");

      ptr[len    ] = '\n';
      ptr[len + 1] = '\0';

      /* åɽ indicate message */
      fputs(ptr, stderr);

      fprintf(stderr, "\033[0m"); /* ̾ɽ conventional indication */
    }else if(*buf & DUMMY);
    else if(*buf & CONTROL){
      ptr[len] = '\0';
      if(0 == strcmp(ptr, "quit"))
        break;
    }else{
      fprintf(stderr, "Invalid packet.\n");
      if(10 < ++count)
        return;
    }
  }

  fclose(snd);
  free(buf);
  free(cry);
  free(cmp);
}

/* consoleʸɤ߹ transmit string from console */
void        tra_msg(int sockfd, OPTION*opt)
{
  char*       buf;  /* buffer for transmission */
  char*       ptr;  /* pointer for data to be transmitted */
  int         len;

  buf = malloc(HED_LEN + MSG_LEN);
  if(NULL == buf){
    fprintf(stderr, "malloc() failure.\n");
    exit(EXIT_FAILURE);
  }

  while(1){
    /* promptɽ indicate prompt */
    if(opt->chat)
      fprintf(stderr, "massage to be transmitted(type 'help' to print usage): ");
    else
      fprintf(stderr, "command(type 'help' to print usage): ");

    ptr = buf + HED_LEN;

    /* åդ accept message */
    if(NULL == fgets(ptr, MSG_LEN, stdin))
      break;

    /* åĹ get length of message */
    len = strlen(ptr);

    /* ʸԤʤоõ delete if last-charactor is newline */
    if('\n' == ptr[len - 1]){
      ptr[len - 1] = '\0';
      len--;
    }

    if(0 == strcmp(ptr, "quit"))
      /* quitäheaderCONTROL˥åȤ */
      *buf = CONTROL;
    else
    if(0 == strcmp(ptr, "chat")){
      /* åȵǽͭˤ enable chat function */
      if(!opt->chat){
        opt->chat = TRUE;
        strcpy(ptr, "chat enabled.");
        len = strlen("chat enabled.");
      }
    }else
    if(0 == strcmp(ptr, "help"))
      /* usageɽ print usage */
      fprintf(stderr,
              "quit: disconnect\n"
              "chat: enable chat function\n"
              "help: print this message\n");
    else if(opt->chat)
      *buf = MESSAGE;
    else{
      fprintf(stderr, "Invalit command.\n");
      continue;
    }


    /* length */
    buf[1] =  len        & 0xff;
    buf[2] = (len >>  8) & 0xff;
    buf[3] = (len >> 16) & 0xff;
    buf[4] = (len >> 24) & 0xff;

    /* message transmit message */
    if((opt->chat || *buf & CONTROL)
    && writen(sockfd, buf, HED_LEN + len) < 0)
      fprintf(stderr, "transmit error\n");

    /* quitä齪λ quit */
    if(0 == strcmp(buf + HED_LEN, "quit"))
      return;
  }

  free(buf);
}

void        tra_snd(int sockfd, OPTION*opt)
{
  char*       buf;  /* buffer for transmission */
  char*       cry;  /* buffer for encryption */
  char*       cmp;  /* buffer for compression */
  char*       ptr;  /* pointer for data to be transmitted */
  FILE*       snd;  /* file pointer for sound device */
  int         len;  /* length */
  int         max_len;  /* maximum length of sound data */
  int         i, j;
  const_des_cblock key = DES_KEY;

  /* Ĺ׻ calculate maximum length */
  max_len = opt->freq * opt->channels * opt->bits / 8 * opt->interval / 1000;

  buf = malloc(HED_LEN + max_len);
  cry = malloc(max_len);
  cmp = malloc(max_len);
  if(NULL == buf || NULL == cry || NULL == cmp){
    fprintf(stderr, "malloc() failure.\n");
    exit(EXIT_FAILURE);
  }

  /* open & initialize sound device (read mode) */
  snd = init_dsp("/dev/dsp", "r", opt->channels, opt->freq, opt->bits);

  /* δĶ transmit environment */
  *buf = CONTROL;
  ptr = buf + HED_LEN;

  snprintf(ptr, max_len,
           "%d %d %d %d",
           opt->freq, opt->channels, opt->bits, opt->interval);

  len = strlen(ptr);

  buf[1] =  len        & 0xff;
  buf[2] = (len >>  8) & 0xff;
  buf[3] = (len >> 16) & 0xff;
  buf[4] = (len >> 24) & 0xff;

  /* packet transmit packet */
  if(writen(sockfd, buf, HED_LEN + len) < 0)
    fprintf(stderr, "transmit error\n");

j=0;
  while(1){
    ptr = buf + HED_LEN;
    len = max_len;

    /* bufϿ record to buf */
    input_buf(snd, ptr, max_len, opt->channels, opt->bits);

    /* headerSOUND˥åȤ set header */
    *buf = SOUND;

    /* ̤ compress */
    i = comp(ptr, max_len, cmp, max_len, 9);

    if(0 <= i && i < max_len){ /* ̤ˤꥵʤä */
      *buf |= COMPRESSED;
      len = i;
      ptr = cmp;
    }

    /* Ψɽ indicate compression ratio */
    if(opt->debug && 20 < j++){
      fprintf(stderr,
              "before: %d, after: %d, ratio: %f\n",
              max_len, len, (double)len / max_len);
      j = 0;
    }

    /* DESǰŹ沽 encrypt by DES */
    *buf |= ENCRYPTED;
    my_encrypt(key, ptr, cry, len);
    ptr = cry;

    /* length */
    buf[1] =  len        & 0xff;
    buf[2] = (len >>  8) & 0xff;
    buf[3] = (len >> 16) & 0xff;
    buf[4] = (len >> 24) & 0xff;

    /* header transmit header */
    if(writen(sockfd, buf, HED_LEN) < 0)
      fprintf(stderr, "transmit error\n");

    /*  transmit sound */
    if(writen(sockfd, ptr, len) < 0)
      fprintf(stderr, "transmit error\n");
  }

  free(buf);
  free(cry);
  free(cmp);
}

void        phone(int sockfd, OPTION*opt)
{
  int         tra_msg_pid; /* process ID of child process for message transmittion */
  int         tra_snd_pid; /* process ID of child process for sound transmittion */

  /* create processes */
  tra_msg_pid = fork();
  tra_snd_pid = fork();
  if(tra_msg_pid < 0 || tra_snd_pid < 0){
    fprintf(stderr, "fork() failure.\n");
    exit(EXIT_FAILURE);
  }

  if(0 == tra_msg_pid)
    tra_msg(sockfd, opt);
  else if(0 == tra_snd_pid)
    tra_snd(sockfd, opt);
  else
    rcv(sockfd, opt);

  kill(tra_msg_pid, SIGTERM);
  kill(tra_snd_pid, SIGTERM);
}

==> phone-1.0.5.tar.gz: phone-1.0.5/communicate.c <==
#include    <netdb.h>
#include    <signal.h>
#include    <stdio.h>
#include    <stdlib.h>
#include    <time.h>
#include    <netinet/in.h>
#include    <sys/types.h>
#include    <sys/socket.h>
#include    <sys/ioctl.h>
#include    <openssl/opensslconf.h>
#include    <openssl/des.h>

#include    "systype.h"

#ifdef      LINUX
#include    <linux/soundcard.h>
#endif

#ifdef      BSD
#include    <soundcard.h>
#endif

#include    "defs.h"
#include    "sock_rw.h"
#include    "dsp.h"
#include    "compress.h"

void        print_time(FILE*fp)
{
  char        buf[64];
  time_t      t;
  int         len;

  /*  */
  time(&t);

  /* ʸѴ */
  ctime_r(&t, buf);

  /* ʸԤʤоõ */
  len = strlen(buf);
  if('\n' == buf[len - 1])
    buf[len - 1] = '\0';

  /* 񤭽Ф */
  fprintf(fp, "%s", buf);
}

void        rcv(int sockfd, OPTION*opt)
{
  char*       buf;  /* buffer for receiving */
  char*       cry;  /* buffer for decryption */
  char*       cmp;  /* buffer for decompression */
  char*       ptr;
  FILE*       snd;
  int         count = 0;
  int         len;
  int         max_len;
  const_des_cblock key = DES_KEY;

  max_len = opt->freq * opt->channels * opt->bits / 8 * opt->interval / 1000;

  buf = malloc(HED_LEN + max(MSG_LEN, max_len) + 1);
  if(NULL == buf){
    fprintf(stderr, "malloc() failure.\n");
    exit(EXIT_FAILURE);
  }

  /* header receive header */
  *buf = 0;
  if(readn(sockfd, buf, HED_LEN) < 0)
    fprintf(stderr, "error\n");

  /* lengthФ */
  len  =  buf[1] & 0xff;
  len |= (buf[2] & 0xff) <<  8;
  len |= (buf[3] & 0xff) << 16;
  len |= (buf[4] & 0xff) << 24;

  ptr = buf + HED_LEN;

  /* Ĥ receive remaing data */
  if(readn(sockfd, ptr, len) < 0)
    fprintf(stderr, "error\n");

  /* δĶ򥻥å */
  opt->freq = atoi(ptr);
  while(' ' != *ptr++);
  opt->channels = atoi(ptr);
  while(' ' != *ptr++);
  opt->bits = atoi(ptr);
  while(' ' != *ptr++);
  opt->interval = atoi(ptr);

print_options(stderr, opt);

  max_len = opt->freq * opt->channels * opt->bits / 8 * opt->interval / 1000;
  free(buf);
  buf = malloc(HED_LEN + max(MSG_LEN, max_len) + 1);
  cry = malloc(max(MSG_LEN, max_len) + 1);
  cmp = malloc(max(MSG_LEN, max_len) + 1);
  if(NULL == buf || NULL == cry || NULL == cmp){
    fprintf(stderr, "malloc() failure.\n");
    exit(EXIT_FAILURE);
  }

  /* open & initialize sound device (write mode) */
  snd = init_dsp(DEV, "w", opt->sys_channels, opt->freq, opt->sys_bits);

  fflush(stdin);
  fflush(stdout);

  while(1){
    /* header receive header */
    *buf = 0;
    if(readn(sockfd, buf, HED_LEN) < 0)
      fprintf(stderr, "error\n");

    /* lengthФ */
    len  =  buf[1] & 0xff;
    len |= (buf[2] & 0xff) <<  8;
    len |= (buf[3] & 0xff) << 16;
    len |= (buf[4] & 0xff) << 24;

    ptr = buf + HED_LEN;

    /* Ĥ receive remaing data */
    if(readn(sockfd, ptr, len) < 0)
      fprintf(stderr, "error\n");

    /* Ź沽Ƥ椹 decrypt if encrypted */
    if(*buf & ENCRYPTED){
      my_decrypt(key, ptr, cry, max_len);
      ptr = cry;
    }

    /* ̤Ƥвह decompress if compressed */
    if(*buf & COMPRESSED){
      decomp(ptr, len, cmp, max_len);
      ptr = cmp;
    }

    /* packet analize packet */
    if(*buf & SOUND)
      output_buf(snd, ptr, max_len, opt);
    else if(*buf & MESSAGE){
      fprintf(stderr,
              "\033[2K"   /* ΤԤõ delete current line */
              "\n"        /*  newline */
              "\033M"     /* 1Ծ˰ư */
              "\033[7m"); /* ȿžɽ reversed indication */

      /* ߤλɽ print current time */
      print_time(stderr);

      fprintf(stderr,
              " massage received: ");

      ptr[len    ] = '\n';
      ptr[len + 1] = '\0';

      /* åɽ indicate message */
      fputs(ptr, stderr);

      fprintf(stderr, "\033[0m"); /* ̾ɽ conventional indication */
    }else if(*buf & DUMMY);
    else if(*buf & CONTROL){
      ptr[len] = '\0';
      if(0 == strcmp(ptr, "quit"))
        break;
    }else{
      fprintf(stderr, "Invalid packet.\n");
      if(10 < ++count)
        return;
    }
  }

  fclose(snd);
  free(buf);
  free(cry);
  free(cmp);
}

/* consoleʸɤ߹ transmit string from console */
void        tra_msg(int sockfd, OPTION*opt)
{
  char*       buf;  /* buffer for transmission */
  char*       ptr;  /* pointer for data to be transmitted */
  int         len;

  buf = malloc(HED_LEN + MSG_LEN);
  if(NULL == buf){
    fprintf(stderr, "malloc() failure.\n");
    exit(EXIT_FAILURE);
  }

  while(1){
    /* promptɽ indicate prompt */
    if(opt->chat)
      fprintf(stderr, "massage to be transmitted(type 'help' to print usage): ");
    else
      fprintf(stderr, "command(type 'help' to print usage): ");

    ptr = buf + HED_LEN;

    /* åդ accept message */
    if(NULL == fgets(ptr, MSG_LEN, stdin))
      break;

    /* åĹ get length of message */
    len = strlen(ptr);

    /* ʸԤʤоõ delete if last-charactor is newline */
    if('\n' == ptr[len - 1]){
      ptr[len - 1] = '\0';
      len--;
    }

    if(0 == strcmp(ptr, "quit"))
      /* quitäheaderCONTROL˥åȤ */
      *buf = CONTROL;
    else
    if(0 == strcmp(ptr, "chat")){
      /* åȵǽͭˤ enable chat function */
      if(!opt->chat){
        opt->chat = TRUE;
        strcpy(ptr, "chat enabled.");
        len = strlen("chat enabled.");
      }
    }else
    if(0 == strcmp(ptr, "help"))
      /* usageɽ print usage */
      fprintf(stderr,
              "quit: disconnect\n"
              "chat: enable chat function\n"
              "help: print this message\n");
    else if(opt->chat)
      *buf = MESSAGE;
    else{
      fprintf(stderr, "Invalit command.\n");
      continue;
    }


    /* length */
    buf[1] =  len        & 0xff;
    buf[2] = (len >>  8) & 0xff;
    buf[3] = (len >> 16) & 0xff;
    buf[4] = (len >> 24) & 0xff;

    /* message transmit message */
    if((opt->chat || *buf & CONTROL)
    && writen(sockfd, buf, HED_LEN + len) < 0)
      fprintf(stderr, "transmit error\n");

    /* quitä齪λ quit */
    if(0 == strcmp(buf + HED_LEN, "quit"))
      return;
  }

  free(buf);
}

void        tra_snd(int sockfd, OPTION*opt)
{
  char*       buf;  /* buffer for transmission */
  char*       cry;  /* buffer for encryption */
  char*       cmp;  /* buffer for compression */
  char*       ptr;  /* pointer for data to be transmitted */
  FILE*       snd;  /* file pointer for sound device */
  int         len;  /* length */
  int         max_len;  /* maximum length of sound data */
  int         i, j;
  const_des_cblock key = DES_KEY;

  /* Ĺ׻ calculate maximum length */
  max_len = opt->freq * opt->channels * opt->bits / 8 * opt->interval / 1000;

  buf = malloc(HED_LEN + max_len);
  cry = malloc(max_len);
  cmp = malloc(max_len);
  if(NULL == buf || NULL == cry || NULL == cmp){
    fprintf(stderr, "malloc() failure.\n");
    exit(EXIT_FAILURE);
  }

  /* open & initialize sound device (read mode) */
  snd = init_dsp(DEV, "r", opt->sys_channels, opt->sys_freq, opt->sys_bits);

  /* δĶ transmit environment */
  *buf = CONTROL;
  ptr = buf + HED_LEN;

  snprintf(ptr, max_len,
           "%d %d %d %d",
           opt->freq, opt->channels, opt->bits, opt->interval);

  len = strlen(ptr);

  buf[1] =  len        & 0xff;
  buf[2] = (len >>  8) & 0xff;
  buf[3] = (len >> 16) & 0xff;
  buf[4] = (len >> 24) & 0xff;

  /* packet transmit packet */
  if(writen(sockfd, buf, HED_LEN + len) < 0)
    fprintf(stderr, "transmit error\n");

  j=0;
  while(1){
    ptr = buf + HED_LEN;
    len = max_len;

    /* bufϿ record to buf */
    input_buf(snd, ptr, max_len, opt);

    /* headerSOUND˥åȤ set header */
    *buf = SOUND;

    /* ̤ compress */
    i = comp(ptr, max_len, cmp, max_len, 9);

    if(0 <= i && i < max_len){ /* ̤ˤꥵʤä */
      *buf |= COMPRESSED;
      len = i;
      ptr = cmp;
    }

    /* Ψɽ indicate compression ratio */
    if(opt->debug && 20 < j++){
      fprintf(stderr,
              "before: %d, after: %d, ratio: %f\n",
              max_len, len, (double)len / max_len);
      j = 0;
    }

    /* DESǰŹ沽 encrypt by DES */
    *buf |= ENCRYPTED;
    my_encrypt(key, ptr, cry, len);
    ptr = cry;

    /* length */
    buf[1] =  len        & 0xff;
    buf[2] = (len >>  8) & 0xff;
    buf[3] = (len >> 16) & 0xff;
    buf[4] = (len >> 24) & 0xff;

    /* header transmit header */
    if(writen(sockfd, buf, HED_LEN) < 0)
      fprintf(stderr, "transmit error\n");

    /*  transmit sound */
    if(writen(sockfd, ptr, len) < 0)
      fprintf(stderr, "transmit error\n");
  }

  free(buf);
  free(cry);
  free(cmp);
}

void        phone(int sockfd, OPTION*opt)
{
  int         tra_msg_pid; /* process ID of child process for message transmittion */
  int         tra_snd_pid; /* process ID of child process for sound transmittion */

  /* create processes */
  tra_msg_pid = fork();
  tra_snd_pid = fork();
  if(tra_msg_pid < 0 || tra_snd_pid < 0){
    fprintf(stderr, "fork() failure.\n");
    exit(EXIT_FAILURE);
  }

  if(0 == tra_msg_pid)
    tra_msg(sockfd, opt);
  else if(0 == tra_snd_pid)
    tra_snd(sockfd, opt);
  else
    rcv(sockfd, opt);

  kill(tra_msg_pid, SIGTERM);
  kill(tra_snd_pid, SIGTERM);
}
